LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY SignIntegerDividerN IS 
	GENERIC(
		N  : integer := 32
	);
	PORT(
		clk      :IN  std_logic;
		go       :IN  std_logic;
		Abus     :IN  std_logic_vector(N-1 DOWNTO 0);
		Bbus     :IN  std_logic_vector(N-1 DOWNTO 0);
		ready    :OUT std_logic;
		Qbus     :OUT std_logic_vector(N-1 DOWNTO 0);
		Rbus     :OUT std_logic_vector(N-1 DOWNTO 0)
	);
END ENTITY SignIntegerDividerN;


ARCHITECTURE RTL OF SignIntegerDividerN IS
	SIGNAL selMux     : std_logic;
	SIGNAL loadInputA : std_logic;
	SIGNAL loadInputB : std_logic;
	SIGNAL rsltCMP    : std_logic;
	SIGNAL BIsZero    : std_logic;
	SIGNAL RisNotZero : std_logic;
	SIGNAL QisNotZero : std_logic;
	SIGNAL BisNotZero : std_logic;
	SIGNAL parLoadR   : std_logic;
	SIGNAL parLoadQ   : std_logic;
	SIGNAL upCount    : std_logic;
	SIGNAL rstCount   : std_logic;
	SIGNAL outMux     : std_logic_vector(N-2 DOWNTO 0);
	SIGNAL ASubB      : std_logic_vector(N-2 DOWNTO 0);
	SIGNAL A          : std_logic_vector(N-2 DOWNTO 0);
	SIGNAL B          : std_logic_vector(N-2 DOWNTO 0);
	SIGNAL Q          : std_logic_vector(N-2 DOWNTO 0);

	COMPONENT ComparatorN IS
		GENERIC( N : integer := N-1 );
		PORT(
			A       :IN  std_logic_vector(N-1 DOWNTO 0);
			B       :IN  std_logic_vector(N-1 DOWNTO 0);
			result  :OUT std_logic
			);
	END COMPONENT;
	COMPONENT CounterN IS
		GENERIC( N : integer := N-1 );
		PORT(
			clk        :IN  std_logic;
			rstCount   :IN  std_logic;
			upCount    :IN  std_logic;
			parOut     :OUT std_logic_vector(N-1 DOWNTO 0)
			);
	END COMPONENT;
	COMPONENT Multiplexer2to1N IS
		GENERIC( N : integer := N-1 );
		PORT(
			A       :IN  std_logic_vector(N-1 DOWNTO 0);
			B       :IN  std_logic_vector(N-1 DOWNTO 0);
			sel     :In  std_logic;
			output  :OUT std_logic_vector(N-1 DOWNTO 0)
		);
	END COMPONENT;	
	COMPONENT SimpleRegisterN IS
		GENERIC( N : integer := N-1 );
		PORT(
			clk        :IN  std_logic;
			parLoad    :IN  std_logic;
			parIn      :In  std_logic_vector(N-1 DOWNTO 0);
			parOut     :OUT std_logic_vector(N-1 DOWNTO 0)
			);
	END COMPONENT;
	COMPONENT SubtractorN IS
		GENERIC( N : integer := N-1 );
		PORT(
			A       :IN  std_logic_vector(N-1 DOWNTO 0);
			B       :IN  std_logic_vector(N-1 DOWNTO 0);
			result  :OUT std_logic_vector(N-1 DOWNTO 0)
			);
	END COMPONENT;
	COMPONENT TriStateN IS
		GENERIC( N : integer := N-1 );
		PORT(
			parLoad    :IN  std_logic;
			parIn      :IN  std_logic_vector(N-1 DOWNTO 0);
			parOut     :OUT std_logic_vector(N-1 DOWNTO 0);
			isNotZero  :OUT std_logic
			);
	END COMPONENT;	
	COMPONENT FSM IS
		PORT(
			clk        :IN  std_logic;
			go         :IN  std_logic;
			rsltCMP    :IN  std_logic;
			BIsZero    :IN  std_logic;
			ready      :OUT std_logic;
			sel        :OUT std_logic;
			loadInputA :OUT std_logic;
			loadInputB :OUT std_logic;
			parLoadR   :OUT std_logic;
			parLoadQ   :OUT std_logic;
			upCount    :OUT std_logic;
			rstCount   :OUT std_logic
			);
	END COMPONENT;
BEGIN
	regA        :SimpleRegisterN   GENERIC MAP(N-1)  PORT MAP(clk, loadInputA, outmux, A);
	regB        :SimpleRegisterN   GENERIC MAP(N-1)  PORT MAP(clk, loadInputB, Bbus(N-2 DOWNTO 0), B);
	mux2to1     :Multiplexer2to1N  GENERIC MAP(N-1)  PORT MAP(Abus(N-2 DOWNTO 0), ASubB, selMux, outMux);
	subUnit     :SubtractorN       GENERIC MAP(N-1)  PORT MAP(A, B, ASubB);
	CMPUnit     :ComparatorN       GENERIC MAP(N-1)  PORT MAP(A, B, rsltCMP);
	triStateR   :TriStateN         GENERIC MAP(N-1)  PORT MAP(parLoadR, A, Rbus(N-2 DOWNTO 0), RisNotZero);
	triStateQ   :TriStateN         GENERIC MAP(N-1)  PORT MAP(parLoadQ, Q, Qbus(N-2 DOWNTO 0), QisNotZero);
	triBIsZero  :TriStateN         GENERIC MAP(N-1)  PORT MAP(loadInputB,  Bbus(N-2 DOWNTO 0), OPEN, BisNotZero);
	counterUnit :CounterN          GENERIC MAP(N-1)  PORT MAP(clk, rstCount, upCount, Q);
	controller  :FSM               PORT MAP(clk, go, rsltCMP, BIsZero, ready, selMux, loadInputA, loadInputB, parLoadR, parLoadQ, upCount, rstCount);
	BIsZero <= NOT(BisNotZero);
	Qbus(N-1) <= QisNotZero AND (Abus(N-1) XOR Bbus(N-1));
	Rbus(N-1) <= RisNotZero AND Abus(N-1);
END RTL;


