LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_textio.ALL;
USE std.textio.ALL;
USE std.env.ALL;

ENTITY tester IS END ENTITY;

ARCHITECTURE timed OF tester IS

	CONSTANT N     : integer := 32; 
	CONSTANT PCLK  : time := 4ns;      
	SIGNAL   clk   : std_logic := '0';
	SIGNAL   go    : std_logic := '0';
	SIGNAL   Abus  : std_logic_vector(N-1 DOWNTO 0);
	SIGNAL   Bbus  : std_logic_vector(N-1 DOWNTO 0);
	SIGNAL   ready : std_logic;
	SIGNAL   Qbus  : std_logic_vector(N-1 DOWNTO 0);
	SIGNAL   Rbus  : std_logic_vector(N-1 DOWNTO 0);

	COMPONENT SignIntegerDividerN IS
		GENERIC(
			N  : integer := N
		);
		PORT(
			clk      :IN  std_logic;
			go       :IN  std_logic;
			Abus     :IN  std_logic_vector(N-1 DOWNTO 0);
			Bbus     :IN  std_logic_vector(N-1 DOWNTO 0);
			ready    :OUT std_logic;
			Qbus     :OUT std_logic_vector(N-1 DOWNTO 0);
			Rbus     :OUT std_logic_vector(N-1 DOWNTO 0)
		);
	END COMPONENT;
	FILE testData : text OPEN READ_MODE IS "TestData.txt";
BEGIN
	UUT : SignIntegerDividerN GENERIC MAP(N) PORT MAP(clk, go, Abus, Bbus, ready, Qbus, Rbus);
	clk <= NOT clk AFTER PCLK/2 ;

	PROCESS(clk)
		VARIABLE flag        : bit := '1';
		VARIABLE index       : integer := 1;
		VARIABLE inputLine   : line;
		VARIABLE displayLine : line;
		VARIABLE AInput    	 : std_logic_vector(N-1 DOWNTO 0);
		VARIABLE BInput    	 : std_logic_vector(N-1 DOWNTO 0);
		VARIABLE expectedQ 	 : std_logic_vector(N-1 DOWNTO 0);
		VARIABLE expectedR 	 : std_logic_vector(N-1 DOWNTO 0);
		
	BEGIN
		IF((ready = '1' AND clk = '0') OR flag = '1')THEN	
			IF(flag = '0')THEN
					WRITE(displayLine, string'("Test with data : "));
					WRITE(displayLine, index);
					WRITELINE(output, displayLine);
					index := index + 1;
				IF(expectedQ /= Qbus OR expectedR /= Rbus)THEN
					WRITE(displayLine, string'("A = "));
					WRITE(displayLine, Abus);
					WRITE(displayLine, string'(" B = "));
					WRITE(displayLine, Bbus);				
					WRITELINE(output, displayLine);
					IF(expectedQ /= Qbus)THEN 
						WRITE(displayLine, string'("Q is wrong !!!"));
						WRITELINE(output, displayLine);
					END IF;
					WRITE(displayLine, string'("Q         = "));
					WRITE(displayLine, Qbus);
					WRITELINE(output, displayLine);
					WRITE(displayLine, string'("expectedQ = "));
					WRITE(displayLine, expectedQ);
					WRITELINE(output, displayLine);
					IF(expectedR /= Rbus)THEN
						WRITE(displayLine, string'("R is wrong !!!"));
						WRITELINE(output, displayLine);
					END IF;
					WRITE(displayLine, string'("R         = "));
					WRITE(displayLine, Rbus);
					WRITELINE(output, displayLine);
					WRITE(displayLine, string'("expectedR = "));
					WRITE(displayLine, expectedR);
					WRITELINE(output, displayLine);
					WRITE(displayLine, string'("At time "));
					WRITE(displayLine, now);
					WRITELINE(output, displayLine);
				END IF;
			END IF;
			IF(NOT(ENDFILE(testData)))THEN
				READLINE(testData, inputLine);
				READ(inputLine, AInput);
				READLINE(testData, inputLine);
				READ(inputLine, BInput);
				READLINE(testData, inputLine);
				READ(inputLine, expectedQ);
				READLINE(testData, inputLine);
				READ(inputLine, expectedR);
				Abus <= AInput AFTER PCLK;
				Bbus <= BInput AFTER PCLK;
				go <= '0', '1' AFTER PCLK, '0' AFTER 2*PCLK;
			ELSE
				finish(0);
			END IF;
		END IF;
		flag := '0';
	END PROCESS;
	


END ARCHITECTURE timed;